@@ -1,20 +1,21 @@
-/* Copyright (C) 2012, 2013 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 2012-2013,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -24,24 +25,28 @@
 #endif
 
 #include <assert.h>
+#include <setjmp.h>
 
-#include "libguile/_scm.h"
-#include "libguile/control.h"
-#include "libguile/eval.h"
-#include "libguile/fluids.h"
-#include "libguile/dynstack.h"
+#include "control.h"
+#include "eval.h"
+#include "fluids.h"
+#include "variable.h"
+#include "threads.h"
+
+#include "dynstack.h"
 
 
 
 
-#define PROMPT_WORDS 5
+#define PROMPT_WORDS 6
 #define PROMPT_KEY(top) (SCM_PACK ((top)[0]))
-#define PROMPT_FP(top) ((scm_t_ptrdiff) ((top)[1]))
+#define PROMPT_FP(top) ((ptrdiff_t) ((top)[1]))
 #define SET_PROMPT_FP(top, fp) do { top[1] = (scm_t_bits)(fp); } while (0)
-#define PROMPT_SP(top) ((scm_t_ptrdiff) ((top)[2]))
+#define PROMPT_SP(top) ((ptrdiff_t) ((top)[2]))
 #define SET_PROMPT_SP(top, sp) do { top[2] = (scm_t_bits)(sp); } while (0)
-#define PROMPT_IP(top) ((scm_t_uint32 *) ((top)[3]))
-#define PROMPT_JMPBUF(top) ((scm_i_jmp_buf *) ((top)[4]))
+#define PROMPT_VRA(top) ((uint32_t *) ((top)[3]))
+#define PROMPT_MRA(top) ((uint8_t *) ((top)[4]))
+#define PROMPT_JMPBUF(top) ((jmp_buf *) ((top)[5]))
 
 #define WINDER_WORDS 2
 #define WINDER_PROC(top) ((scm_t_guard) ((top)[0]))
@@ -192,8 +197,8 @@ void
 scm_dynstack_push_prompt (scm_t_dynstack *dynstack,
                           scm_t_dynstack_prompt_flags flags,
                           SCM key,
-                          scm_t_ptrdiff fp_offset, scm_t_ptrdiff sp_offset,
-                          scm_t_uint32 *ip, scm_i_jmp_buf *registers)
+                          ptrdiff_t fp_offset, ptrdiff_t sp_offset,
+                          uint32_t *vra, uint8_t *mra, jmp_buf *registers)
 {
   scm_t_bits *words;
 
@@ -202,8 +207,9 @@ scm_dynstack_push_prompt (scm_t_dynstack *dynstack,
   words[0] = SCM_UNPACK (key);
   words[1] = (scm_t_bits) fp_offset;
   words[2] = (scm_t_bits) sp_offset;
-  words[3] = (scm_t_bits) ip;
-  words[4] = (scm_t_bits) registers;
+  words[3] = (scm_t_bits) vra;
+  words[4] = (scm_t_bits) mra;
+  words[5] = (scm_t_bits) registers;
 }
 
 void
@@ -290,7 +296,7 @@ scm_dynstack_capture (scm_t_dynstack *dynstack, scm_t_bits *item)
 }
 
 void
-scm_dynstack_relocate_prompts (scm_t_dynstack *dynstack, scm_t_ptrdiff base)
+scm_dynstack_relocate_prompts (scm_t_dynstack *dynstack, ptrdiff_t base)
 {
   scm_t_bits *walk;
 
@@ -495,8 +501,8 @@ scm_dynstack_unwind_fork (scm_t_dynstack *dynstack, scm_t_dynstack *branch)
 scm_t_bits*
 scm_dynstack_find_prompt (scm_t_dynstack *dynstack, SCM key,
                           scm_t_dynstack_prompt_flags *flags,
-                          scm_t_ptrdiff *fp_offset, scm_t_ptrdiff *sp_offset,
-                          scm_t_uint32 **ip, scm_i_jmp_buf **registers)
+                          ptrdiff_t *fp_offset, ptrdiff_t *sp_offset,
+                          uint32_t **vra, uint8_t **mra, jmp_buf **registers)
 {
   scm_t_bits *walk;
 
@@ -514,8 +520,10 @@ scm_dynstack_find_prompt (scm_t_dynstack *dynstack, SCM key,
             *fp_offset = PROMPT_FP (walk);
           if (sp_offset)
             *sp_offset = PROMPT_SP (walk);
-          if (ip)
-            *ip = PROMPT_IP (walk);
+          if (vra)
+            *vra = PROMPT_VRA (walk);
+          if (mra)
+            *mra = PROMPT_MRA (walk);
           if (registers)
             *registers = PROMPT_JMPBUF (walk);
           return walk;
@@ -576,8 +584,8 @@ scm_dynstack_find_old_fluid_value (scm_t_dynstack *dynstack, SCM fluid,
 
 void
 scm_dynstack_wind_prompt (scm_t_dynstack *dynstack, scm_t_bits *item,
-                          scm_t_ptrdiff base_fp_offset,
-                          scm_i_jmp_buf *registers)
+                          ptrdiff_t base_fp_offset,
+                          jmp_buf *registers)
 {
   scm_t_bits tag = SCM_DYNSTACK_TAG (item);
 
@@ -589,7 +597,8 @@ scm_dynstack_wind_prompt (scm_t_dynstack *dynstack, scm_t_bits *item,
                             PROMPT_KEY (item),
                             PROMPT_FP (item) + base_fp_offset,
                             PROMPT_SP (item) + base_fp_offset,
-                            PROMPT_IP (item),
+                            PROMPT_VRA (item),
+                            PROMPT_MRA (item),
                             registers);
 }
 
@@ -664,9 +673,3 @@ scm_dynstack_unwind_dynamic_state (scm_t_dynstack *dynstack,
   clear_scm_t_bits (words, len);
 }
 
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
