@@ -1,21 +1,21 @@
-/* Copyright (C) 1995-2004, 2006-2015, 2017, 2019
- *   Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-2004,2006-2015,2017-2020
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -44,20 +44,33 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/select.h>
-
 #include <full-write.h>
 
-#include "libguile/_scm.h"
-#include "libguile/fdes-finalizers.h"
-#include "libguile/strings.h"
-#include "libguile/validate.h"
-#include "libguile/gc.h"
-#include "libguile/posix.h"
-#include "libguile/dynwind.h"
-#include "libguile/hashtab.h"
+#include "async.h"
+#include "boolean.h"
+#include "dynwind.h"
+#include "extensions.h"
+#include "fdes-finalizers.h"
+#include "filesys.h"
+#include "fluids.h"
+#include "gc.h"
+#include "gsubr.h"
+#include "hashtab.h"
+#include "keywords.h"
+#include "modules.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "ports-internal.h"
+#include "posix.h"
+#include "read.h"
+#include "strings.h"
+#include "symbols.h"
+#include "syscalls.h"
+#include "variable.h"
+#include "version.h"
+
+#include "fports.h"
 
-#include "libguile/fports.h"
-#include "libguile/ports-internal.h"
 
 #if SIZEOF_OFF_T == SIZEOF_INT
 #define OFF_T_MAX  INT_MAX
@@ -388,6 +401,29 @@ SCM_DEFINE (scm_i_open_file, "open-file", 2, 0, 1,
 
 /* Building Guile ports from a file descriptor.  */
 
+int
+scm_i_fdes_is_valid (int fdes, long mode_bits)
+{
+#ifdef F_GETFL
+  int flags = fcntl (fdes, F_GETFL, 0);
+  if (flags == -1)
+    return 0;
+  flags &= O_ACCMODE;
+  if (flags == O_RDWR)
+    return 1;
+  if (flags != O_WRONLY && (mode_bits & SCM_WRTNG))
+    return 0;
+  if (flags != O_RDONLY && (mode_bits & SCM_RDNG))
+    return 0;
+  return 1;
+#else
+  /* If we don't have F_GETFL, as on mingw, at least we can test that
+     it is a valid file descriptor.  */
+  struct stat st;
+  return fstat (fdes, &st) == 0;
+#endif
+}
+
 /* Build a Scheme port from an open file descriptor `fdes'.
    MODE indicates whether FILE is open for reading or writing; it uses
       the same notation as open-file's second argument.
@@ -402,33 +438,21 @@ scm_i_fdes_to_port (int fdes, long mode_bits, SCM name, unsigned options)
 
   if (options & SCM_FPORT_OPTION_VERIFY)
     {
-      /* Check that the foreign FD is valid and matches the mode
-         bits.  */
-#ifdef F_GETFL
-      int flags = fcntl (fdes, F_GETFL, 0);
-      if (flags == -1)
-        SCM_SYSERROR;
-      flags &= O_ACCMODE;
-      if (flags != O_RDWR
-          && ((flags != O_WRONLY && (mode_bits & SCM_WRTNG))
-              || (flags != O_RDONLY && (mode_bits & SCM_RDNG))))
+      errno = 0;
+      if (!scm_i_fdes_is_valid (fdes, mode_bits))
         {
+          if (errno)
+            SCM_SYSERROR;
           SCM_MISC_ERROR ("requested file mode not available on fdes",
                           SCM_EOL);
         }
-#else
-      /* If we don't have F_GETFL, as on mingw, at least we can test that
-         it is a valid file descriptor.  */
-      struct stat st;
-      if (fstat (fdes, &st) != 0)
-        SCM_SYSERROR;
-#endif
     }
 
   fp = (scm_t_fport *) scm_gc_malloc_pointerless (sizeof (scm_t_fport),
                                                   "file port");
   fp->fdes = fdes;
   fp->options = options;
+  fp->revealed = 0;
 
   port = scm_c_make_port (scm_file_port_type, mode_bits, (scm_t_bits)fp);
   
@@ -715,7 +739,7 @@ scm_init_fports_keywords ()
 static void
 scm_init_ice_9_fports (void)
 {
-#include "libguile/fports.x"
+#include "fports.x"
 }
 
 void
@@ -741,9 +765,3 @@ scm_init_fports ()
   scm_c_define ("%file-port-name-canonicalization",
                 sys_file_port_name_canonicalization);
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
