@@ -1,38 +1,27 @@
-/* Copyright (C) 2006-2014, 2017 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 2006-2014,2017-2019
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #include <alloca.h>
-
-#include "libguile/_scm.h"
-#include "libguile/extensions.h"
-#include "libguile/feature.h"
-#include "libguile/i18n.h"
-#include "libguile/strings.h"
-#include "libguile/chars.h"
-#include "libguile/dynwind.h"
-#include "libguile/validate.h"
-#include "libguile/values.h"
-#include "libguile/threads.h"
-
 #include <locale.h>
 #include <string.h> /* `strcoll ()' */
 #include <ctype.h>  /* `toupper ()' et al. */
@@ -40,6 +29,32 @@
 #include <unicase.h>
 #include <unistr.h>
 
+#include "boolean.h"
+#include "chars.h"
+#include "dynwind.h"
+#include "extensions.h"
+#include "feature.h"
+#include "gsubr.h"
+#include "list.h"
+#include "modules.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "posix.h"  /* for `scm_i_locale_mutex' */
+#include "smob.h"
+#include "strings.h"
+#include "symbols.h"
+#include "syscalls.h"
+#include "threads.h"
+#include "values.h"
+#include "variable.h"
+#include "version.h"
+
+#include "i18n.h"
+
+#ifndef SCM_MAX_ALLOCA
+# define SCM_MAX_ALLOCA 4096 /* Max bytes per string to allocate via alloca */
+#endif
+
 #if defined HAVE_NEWLOCALE && defined HAVE_STRCOLL_L && defined HAVE_USELOCALE
 /* The GNU thread-aware locale API is documented in ``Thread-Aware Locale
    Model, a Proposal'', by Ulrich Drepper:
@@ -57,8 +72,6 @@
 # define USE_GNU_LOCALE_API
 #endif
 
-#include "libguile/posix.h"  /* for `scm_i_locale_mutex' */
-
 /* Use Gnulib's header, which also provides `nl_item' & co.  */
 #include <langinfo.h>
 
@@ -71,6 +84,31 @@ setlocale (int category, const char *name)
 }
 #endif
 
+/* The newlib C library has alternative names for locale constants.  */
+#if HAVE_DECL__NL_NUMERIC_GROUPING
+#define INT_CURR_SYMBOL    _NL_MONETARY_INT_CURR_SYMBOL
+#define MON_DECIMAL_POINT  _NL_MONETARY_MON_DECIMAL_POINT
+#define MON_THOUSANDS_SEP  _NL_MONETARY_MON_THOUSANDS_SEP
+#define MON_GROUPING       _NL_MONETARY_MON_GROUPING
+#define POSITIVE_SIGN      _NL_MONETARY_POSITIVE_SIGN
+#define NEGATIVE_SIGN      _NL_MONETARY_NEGATIVE_SIGN
+#define GROUPING           _NL_NUMERIC_GROUPING
+#define INT_FRAC_DIGITS    _NL_MONETARY_INT_FRAC_DIGITS
+#define FRAC_DIGITS        _NL_MONETARY_FRAC_DIGITS
+#define P_CS_PRECEDES      _NL_MONETARY_P_CS_PRECEDES
+#define P_SEP_BY_SPACE     _NL_MONETARY_P_SEP_BY_SPACE
+#define N_CS_PRECEDES      _NL_MONETARY_N_CS_PRECEDES
+#define N_SEP_BY_SPACE     _NL_MONETARY_N_SEP_BY_SPACE
+#define P_SIGN_POSN        _NL_MONETARY_P_SIGN_POSN
+#define N_SIGN_POSN        _NL_MONETARY_N_SIGN_POSN
+#define INT_P_CS_PRECEDES  _NL_MONETARY_INT_P_CS_PRECEDES
+#define INT_P_SEP_BY_SPACE _NL_MONETARY_INT_P_SEP_BY_SPACE
+#define INT_N_CS_PRECEDES  _NL_MONETARY_INT_N_CS_PRECEDES
+#define INT_N_SEP_BY_SPACE _NL_MONETARY_INT_N_SEP_BY_SPACE
+#define INT_P_SIGN_POSN    _NL_MONETARY_INT_P_SIGN_POSN
+#define INT_N_SIGN_POSN    _NL_MONETARY_INT_N_SIGN_POSN
+#endif
+
 /* Helper stringification macro.  */
 #define SCM_I18N_STRINGIFY(_name)   # _name
 
@@ -718,50 +756,61 @@ SCM_DEFINE (scm_locale_p, "locale?", 1, 0, 0,
    A similar API can be found in MzScheme starting from version 200:
    http://download.plt-scheme.org/chronology/mzmr200alpha14.html .  */
 
-#define SCM_STRING_TO_U32_BUF(s1, c_s1)					\
-  do									\
-    {									\
-      if (scm_i_is_narrow_string (s1))					\
-	{								\
-	  size_t i, len;						\
-	  const char *buf = scm_i_string_chars (s1);			\
-									\
-	  len = scm_i_string_length (s1);				\
-	  c_s1 = alloca (sizeof (scm_t_wchar) * (len + 1));		\
-									\
-	  for (i = 0; i < len; i ++)					\
-	    c_s1[i] = (unsigned char ) buf[i];				\
-	  c_s1[len] = 0;						\
-	}								\
-      else								\
-	c_s1 = (scm_t_wchar *) scm_i_string_wide_chars (s1);		\
+#define SCM_STRING_TO_U32_BUF(str, c_str, c_str_malloc_p)               \
+  do                                                                    \
+    {                                                                   \
+      if (scm_i_is_narrow_string (str))                                 \
+        {                                                               \
+          size_t i, len, bytes;                                         \
+          const char *buf = scm_i_string_chars (str);                   \
+                                                                        \
+          len = scm_i_string_length (str);                              \
+          bytes = (len + 1) * sizeof (scm_t_wchar);                     \
+          c_str_malloc_p = (bytes > SCM_MAX_ALLOCA);                    \
+          c_str = c_str_malloc_p ? malloc (bytes) : alloca (bytes);     \
+                                                                        \
+          for (i = 0; i < len; i ++)                                    \
+            c_str[i] = (unsigned char ) buf[i];                         \
+          c_str[len] = 0;                                               \
+        }                                                               \
+      else                                                              \
+        {                                                               \
+          c_str_malloc_p = 0;                                           \
+          c_str = (scm_t_wchar *) scm_i_string_wide_chars (str);        \
+        }                                                               \
+    } while (0)
+
+#define SCM_CLEANUP_U32_BUF(c_str, c_str_malloc_p)                      \
+  do                                                                    \
+    {                                                                   \
+      if (c_str_malloc_p)                                               \
+        free (c_str);                                                   \
     } while (0)
 
 
-/* Compare UTF-32 strings according to LOCALE.  Returns a negative value if
-   S1 compares smaller than S2, a positive value if S1 compares larger than
+/* Compare strings according to LOCALE.  Returns a negative value if S1
+   compares smaller than S2, a positive value if S1 compares larger than
    S2, or 0 if they compare equal.  */
 static inline int
-compare_u32_strings (SCM s1, SCM s2, SCM locale, const char *func_name) 
+compare_strings (SCM s1, SCM s2, SCM locale, const char *func_name)
 #define FUNC_NAME func_name
 {
   int result;
   scm_t_locale c_locale;
-  scm_t_wchar *c_s1, *c_s2;
+  uint8_t *c_s1, *c_s2;
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (3, locale, c_locale);
 
-  SCM_STRING_TO_U32_BUF (s1, c_s1);
-  SCM_STRING_TO_U32_BUF (s2, c_s2);
+  c_s1 = (uint8_t *) scm_to_utf8_string (s1);
+  c_s2 = (uint8_t *) scm_to_utf8_string (s2);
 
   if (c_locale)
-    RUN_IN_LOCALE_SECTION (c_locale, 
-                           result = u32_strcoll ((const scm_t_uint32 *) c_s1, 
-                                                 (const scm_t_uint32 *) c_s2));
+    RUN_IN_LOCALE_SECTION (c_locale, result = u8_strcoll (c_s1, c_s2));
   else
-    result = u32_strcoll ((const scm_t_uint32 *) c_s1,
-			  (const scm_t_uint32 *) c_s2);
+    result = u8_strcoll (c_s1, c_s2);
+
+  free (c_s1);
+  free (c_s2);
 
-  scm_remember_upto_here_2 (s1, s2);
   scm_remember_upto_here (locale);
   return result;
 }
@@ -779,9 +828,8 @@ locale_language ()
 }
 
 static inline int
-u32_locale_casecoll (const char *func_name, const scm_t_uint32 *c_s1,
-                     const scm_t_uint32 *c_s2,
-		     int *result)
+u32_locale_casecoll (const char *func_name, const uint32_t *c_s1, size_t s1_len,
+                     const uint32_t *c_s2, size_t s2_len, int *result)
 {
   /* Note: Since this is called from `RUN_IN_LOCALE_SECTION', it must note
      make any non-local exit.  */
@@ -789,8 +837,8 @@ u32_locale_casecoll (const char *func_name, const scm_t_uint32 *c_s1,
   int ret;
   const char *loc = locale_language ();
 
-  ret = u32_casecoll (c_s1, u32_strlen (c_s1),
-                      c_s2, u32_strlen (c_s2),
+  ret = u32_casecoll (c_s1, s1_len,
+                      c_s2, s2_len,
                       loc, UNINORM_NFC, result);
 
   return ret == 0 ? ret : errno;
@@ -803,24 +851,31 @@ compare_u32_strings_ci (SCM s1, SCM s2, SCM locale, const char *func_name)
   int result, ret = 0;
   scm_t_locale c_locale;
   scm_t_wchar *c_s1, *c_s2;
+  int c_s1_malloc_p, c_s2_malloc_p;
   SCM_VALIDATE_OPTIONAL_LOCALE_COPY (3, locale, c_locale);
 
-  SCM_STRING_TO_U32_BUF (s1, c_s1);
-  SCM_STRING_TO_U32_BUF (s2, c_s2);
+  size_t s1_len = scm_i_string_length (s1);
+  size_t s2_len = scm_i_string_length (s2);
+
+  SCM_STRING_TO_U32_BUF (s1, c_s1, c_s1_malloc_p);
+  SCM_STRING_TO_U32_BUF (s2, c_s2, c_s2_malloc_p);
 
   if (c_locale)
     RUN_IN_LOCALE_SECTION
       (c_locale,
        ret = u32_locale_casecoll (func_name,
-				  (const scm_t_uint32 *) c_s1,
-				  (const scm_t_uint32 *) c_s2,
+				  (const uint32_t *) c_s1, s1_len,
+				  (const uint32_t *) c_s2, s2_len,
 				  &result));
   else
     ret = u32_locale_casecoll (func_name,
-			       (const scm_t_uint32 *) c_s1,
-			       (const scm_t_uint32 *) c_s2,
+			       (const uint32_t *) c_s1, s1_len,
+			       (const uint32_t *) c_s2, s2_len,
 			       &result);
 
+  SCM_CLEANUP_U32_BUF(c_s1, c_s1_malloc_p);
+  SCM_CLEANUP_U32_BUF(c_s2, c_s2_malloc_p);
+
   if (SCM_UNLIKELY (ret != 0))
     {
       errno = ret;
@@ -847,7 +902,7 @@ SCM_DEFINE (scm_string_locale_lt, "string-locale<?", 2, 1, 0,
   SCM_VALIDATE_STRING (1, s1);
   SCM_VALIDATE_STRING (2, s2);
 
-  result = compare_u32_strings (s1, s2, locale, FUNC_NAME);
+  result = compare_strings (s1, s2, locale, FUNC_NAME);
 
   return scm_from_bool (result < 0);
 }
@@ -866,7 +921,7 @@ SCM_DEFINE (scm_string_locale_gt, "string-locale>?", 2, 1, 0,
   SCM_VALIDATE_STRING (1, s1);
   SCM_VALIDATE_STRING (2, s2);
 
-  result = compare_u32_strings (s1, s2, locale, FUNC_NAME);
+  result = compare_strings (s1, s2, locale, FUNC_NAME);
 
   return scm_from_bool (result > 0);
 }
@@ -944,9 +999,9 @@ SCM_DEFINE (scm_char_locale_lt, "char-locale<?", 2, 1, 0,
   SCM_VALIDATE_CHAR (1, c1);
   SCM_VALIDATE_CHAR (2, c2);
 
-  result = compare_u32_strings (scm_string (scm_list_1 (c1)), 
-                                scm_string (scm_list_1 (c2)), 
-                                locale, FUNC_NAME);
+  result = compare_strings (scm_string (scm_list_1 (c1)),
+                            scm_string (scm_list_1 (c2)),
+                            locale, FUNC_NAME);
 
   return scm_from_bool (result < 0);
 }
@@ -963,9 +1018,9 @@ SCM_DEFINE (scm_char_locale_gt, "char-locale>?", 2, 1, 0,
   SCM_VALIDATE_CHAR (1, c1);
   SCM_VALIDATE_CHAR (2, c2);
 
-  result = compare_u32_strings (scm_string (scm_list_1 (c1)), 
-                                scm_string (scm_list_1 (c2)), 
-                                locale, FUNC_NAME);
+  result = compare_strings (scm_string (scm_list_1 (c1)),
+                            scm_string (scm_list_1 (c2)),
+                            locale, FUNC_NAME);
 
   return scm_from_bool (result > 0);
 }
@@ -1036,16 +1091,16 @@ SCM_DEFINE (scm_char_locale_ci_eq, "char-locale-ci=?", 2, 1, 0,
 /* Locale-dependent alphabetic character mapping.  */
 
 static inline int
-u32_locale_tocase (const scm_t_uint32 *c_s1, size_t len,
-                   scm_t_uint32 **p_c_s2, size_t * p_len2,
-                   scm_t_uint32 *(*func) (const scm_t_uint32 *, size_t,
+u32_locale_tocase (const uint32_t *c_s1, size_t len,
+                   uint32_t **p_c_s2, size_t * p_len2,
+                   uint32_t *(*func) (const uint32_t *, size_t,
                                           const char *, uninorm_t,
-                                          scm_t_uint32 *, size_t *))
+                                          uint32_t *, size_t *))
 {
   /* Note: Since this is called from `RUN_IN_LOCALE_SECTION', it must not
      make any non-local exit.  */
 
-  scm_t_uint32 *ret;
+  uint32_t *ret;
   const char *loc = locale_language ();
 
   /* The first NULL here indicates that no NFC or NFKC normalization
@@ -1055,7 +1110,7 @@ u32_locale_tocase (const scm_t_uint32 *c_s1, size_t len,
 
   if (ret == NULL)
     {
-      *p_c_s2 = (scm_t_uint32 *) NULL;
+      *p_c_s2 = (uint32_t *) NULL;
       *p_len2 = 0;
       return errno;
     }
@@ -1067,15 +1122,15 @@ u32_locale_tocase (const scm_t_uint32 *c_s1, size_t len,
 
 static SCM
 chr_to_case (SCM chr, scm_t_locale c_locale, 
-	     scm_t_uint32 *(*func) (const scm_t_uint32 *, size_t, const char *,
-				    uninorm_t, scm_t_uint32 *, size_t *),
+	     uint32_t *(*func) (const uint32_t *, size_t, const char *,
+				    uninorm_t, uint32_t *, size_t *),
 	     const char *func_name,
 	     int *err)
 #define FUNC_NAME func_name
 {
   int ret;
-  scm_t_uint32 c;
-  scm_t_uint32 *convbuf;
+  uint32_t c;
+  uint32_t *convbuf;
   size_t convlen;
   SCM convchar;
 
@@ -1178,33 +1233,35 @@ SCM_DEFINE (scm_char_locale_titlecase, "char-locale-titlecase", 1, 1, 0,
 
 static SCM
 str_to_case (SCM str, scm_t_locale c_locale,
-	     scm_t_uint32 *(*func) (const scm_t_uint32 *, size_t, const char *,
-				    uninorm_t, scm_t_uint32 *, size_t *),
+	     uint32_t *(*func) (const uint32_t *, size_t, const char *,
+				    uninorm_t, uint32_t *, size_t *),
 	     const char *func_name,
 	     int *err)
 #define FUNC_NAME func_name
 {
   scm_t_wchar *c_str, *c_buf;
-  scm_t_uint32 *c_convstr;
+  uint32_t *c_convstr;
   size_t len, convlen;
-  int ret;
+  int ret, c_str_malloc_p;
   SCM convstr;
 
   len = scm_i_string_length (str);
   if (len == 0)
     return scm_nullstr;
-  SCM_STRING_TO_U32_BUF (str, c_str);
+  SCM_STRING_TO_U32_BUF (str, c_str, c_str_malloc_p);
 
   if (c_locale)
     RUN_IN_LOCALE_SECTION (c_locale, ret =
-                           u32_locale_tocase ((scm_t_uint32 *) c_str, len,
+                           u32_locale_tocase ((uint32_t *) c_str, len,
                                               &c_convstr,
                                               &convlen, func));
   else
     ret =
-      u32_locale_tocase ((scm_t_uint32 *) c_str, len,
+      u32_locale_tocase ((uint32_t *) c_str, len,
                          &c_convstr, &convlen, func);
 
+  SCM_CLEANUP_U32_BUF(c_str, c_str_malloc_p);
+
   scm_remember_upto_here (str);
 
   if (SCM_UNLIKELY (ret != 0))
@@ -1352,7 +1409,7 @@ SCM_DEFINE (scm_locale_string_to_integer, "locale-string->integer",
   else
     result = scm_from_long (c_result);
 
-  return (scm_values (scm_list_2 (result, scm_from_long (c_endptr - c_str))));
+  return scm_values_2 (result, scm_from_long (c_endptr - c_str));
 }
 #undef FUNC_NAME
 
@@ -1396,7 +1453,7 @@ SCM_DEFINE (scm_locale_string_to_inexact, "locale-string->inexact",
   else
     result = scm_from_double (c_result);
 
-  return (scm_values (scm_list_2 (result, scm_from_long (c_endptr - c_str))));
+  return scm_values_2 (result, scm_from_long (c_endptr - c_str));
 }
 #undef FUNC_NAME
 
@@ -1827,7 +1884,7 @@ scm_init_i18n ()
   scm_add_feature ("nl-langinfo");
   define_langinfo_items ();
 
-#include "libguile/i18n.x"
+#include "i18n.x"
 
   /* Initialize the global locale object with a special `locale' SMOB.  */
   /* XXX: We don't define it as `LC_GLOBAL_LOCALE' because of bugs as of
@@ -1847,9 +1904,3 @@ scm_bootstrap_i18n ()
 
 }
 
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
