@@ -1,22 +1,21 @@
-/* Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
- *   2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
- *   2014, 2016, 2017, 2018 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-2014,2016-2019,2021
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -24,18 +23,20 @@
 #  include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <stdio.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <uniconv.h>
+#include <unistd.h>
 
 #ifdef HAVE_SCHED_H
 # include <sched.h>
 #endif
 
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
 #ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -47,44 +48,51 @@
 # endif
 #endif
 
-#include <unistd.h>
-
 #ifdef LIBC_H_WITH_UNISTD_H
-#include <libc.h>
+# include <libc.h>
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
 #ifdef HAVE_PWD_H
-#include <pwd.h>
+# include <pwd.h>
 #endif
 #ifdef HAVE_IO_H
-#include <io.h>
+# include <io.h>
 #endif
 
-#include "libguile/_scm.h"
-#include "libguile/dynwind.h"
-#include "libguile/fports.h"
-#include "libguile/scmsigs.h"
-#include "libguile/feature.h"
-#include "libguile/strings.h"
-#include "libguile/srfi-13.h"
-#include "libguile/srfi-14.h"
-#include "libguile/vectors.h"
-#include "libguile/values.h"
-
-#include "libguile/validate.h"
-#include "libguile/posix.h"
-#include "libguile/gettext.h"
-#include "libguile/threads.h"
-
-
 #ifdef __MINGW32__
 # include "posix-w32.h"
 #endif
 
+#include "async.h"
+#include "bitvectors.h"
+#include "dynwind.h"
+#include "extensions.h"
+#include "feature.h"
+#include "finalizers.h"
+#include "fports.h"
+#include "gettext.h"
+#include "gsubr.h"
+#include "list.h"
+#include "modules.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "scmsigs.h"
+#include "srfi-13.h"
+#include "srfi-14.h"
+#include "strings.h"
+#include "symbols.h"
+#include "syscalls.h"
+#include "threads.h"
+#include "values.h"
+#include "vectors.h"
+#include "version.h"
+
+#if (SCM_ENABLE_DEPRECATED == 1)
+#include "deprecation.h"
+#endif
+
+#include "posix.h"
+
 #if HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
@@ -372,11 +380,11 @@ SCM_DEFINE (scm_getpwuid, "getpw", 0, 1, 0,
   SCM_SIMPLE_VECTOR_SET(result, 3, scm_from_ulong (entry->pw_gid));
   SCM_SIMPLE_VECTOR_SET(result, 4, scm_from_locale_string (entry->pw_gecos));
   if (!entry->pw_dir)
-    SCM_SIMPLE_VECTOR_SET(result, 5, scm_from_locale_string (""));
+    SCM_SIMPLE_VECTOR_SET(result, 5, scm_from_utf8_string (""));
   else
     SCM_SIMPLE_VECTOR_SET(result, 5, scm_from_locale_string (entry->pw_dir));
   if (!entry->pw_shell)
-    SCM_SIMPLE_VECTOR_SET(result, 6, scm_from_locale_string (""));
+    SCM_SIMPLE_VECTOR_SET(result, 6, scm_from_utf8_string (""));
   else
     SCM_SIMPLE_VECTOR_SET(result, 6, scm_from_locale_string (entry->pw_shell));
   return result;
@@ -596,10 +604,10 @@ SCM_DEFINE (scm_getrlimit, "getrlimit", 1, 0, 0,
   if (getrlimit (iresource, &lim) != 0)
     scm_syserror (FUNC_NAME);
 
-  return scm_values (scm_list_2 ((lim.rlim_cur == RLIM_INFINITY) ? SCM_BOOL_F
-                                 : scm_from_long (lim.rlim_cur),
-                                 (lim.rlim_max == RLIM_INFINITY) ? SCM_BOOL_F
-                                 : scm_from_long (lim.rlim_max)));
+  return scm_values_2 ((lim.rlim_cur == RLIM_INFINITY) ? SCM_BOOL_F
+                       : scm_from_long (lim.rlim_cur),
+                       (lim.rlim_max == RLIM_INFINITY) ? SCM_BOOL_F
+                       : scm_from_long (lim.rlim_max));
 }
 #undef FUNC_NAME
 
@@ -1209,6 +1217,31 @@ SCM_DEFINE (scm_execle, "execle", 2, 0, 1,
 #undef FUNC_NAME
 
 #ifdef HAVE_FORK
+
+/* Create a process and perform post-fork cleanups in the child.  */
+static void *
+do_fork (void *ret)
+{
+  pid_t pid = fork ();
+
+  if (pid == 0)
+    {
+      /* The child process must not share its sleep pipe with the
+         parent.  Close it and create a new one.  */
+      int err;
+      scm_thread *t = SCM_I_CURRENT_THREAD;
+
+      close (t->sleep_pipe[0]);
+      close (t->sleep_pipe[1]);
+      err = pipe2 (t->sleep_pipe, O_CLOEXEC);
+      if (err != 0)
+        abort ();
+    }
+
+  * (pid_t *) ret = pid;
+  return NULL;
+}
+
 SCM_DEFINE (scm_fork, "primitive-fork", 0, 0, 0,
             (),
 	    "Creates a new \"child\" process by duplicating the current \"parent\" process.\n"
@@ -1236,7 +1269,9 @@ SCM_DEFINE (scm_fork, "primitive-fork", 0, 0, 0,
         "         further behavior unspecified.  See \"Processes\" in the\n"
         "         manual, for more information.\n"),
        scm_current_warning_port ());
-  pid = fork ();
+
+  scm_without_guile (do_fork, &pid);
+
   if (pid == -1)
     SCM_SYSERROR;
   return scm_from_int (pid);
@@ -1244,6 +1279,36 @@ SCM_DEFINE (scm_fork, "primitive-fork", 0, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_FORK */
 
+#ifdef HAVE_FORK
+/* 'renumber_file_descriptor' is a helper function for 'start_child'
+   below, and is specialized for that particular environment where it
+   doesn't make sense to report errors via exceptions.  It uses dup(2)
+   to duplicate the file descriptor FD, closes the original FD, and
+   returns the new descriptor.  If dup(2) fails, print an error message
+   to ERR and abort.  */
+static int
+renumber_file_descriptor (int fd, int err)
+{
+  int new_fd;
+
+  do
+    new_fd = dup (fd);
+  while (new_fd == -1 && errno == EINTR);
+
+  if (new_fd == -1)
+    {
+      /* At this point we are in the child process before exec.  We
+         cannot safely raise an exception in this environment.  */
+      char *msg = strerror (errno);
+      fprintf (fdopen (err, "a"), "start_child: dup failed: %s\n", msg);
+      _exit (127);  /* Use exit status 127, as with other exec errors. */
+    }
+
+  close (fd);
+  return new_fd;
+}
+#endif /* HAVE_FORK */
+
 #ifdef HAVE_FORK
 #define HAVE_START_CHILD 1
 /* Since Guile uses threads, we have to be very careful to avoid calling
@@ -1295,16 +1360,16 @@ start_child (const char *exec_file, char **exec_argv,
   if (in > 0)
     {
       if (out == 0)
-        do out = dup (out); while (errno == EINTR);
+        out = renumber_file_descriptor (out, err);
       if (err == 0)
-        do err = dup (err); while (errno == EINTR);
+        err = renumber_file_descriptor (err, err);
       do dup2 (in, 0); while (errno == EINTR);
       close (in);
     }
   if (out > 1)
     {
       if (err == 1)
-        do err = dup (err); while (errno == EINTR);
+        err = renumber_file_descriptor (err, err);
       do dup2 (out, 1); while (errno == EINTR);
       close (out);
     }
@@ -1317,12 +1382,11 @@ start_child (const char *exec_file, char **exec_argv,
   execvp (exec_file, exec_argv);
 
   /* The exec failed!  There is nothing sensible to do.  */
-  if (err > 0)
-    {
-      char *msg = strerror (errno);
-      fprintf (fdopen (err, "a"), "In execvp of %s: %s\n",
-               exec_file, msg);
-    }
+  {
+    char *msg = strerror (errno);
+    fprintf (fdopen (2, "a"), "In execvp of %s: %s\n",
+             exec_file, msg);
+  }
 
   /* Use exit status 127, like shells in this case, as per POSIX
      <http://pubs.opengroup.org/onlinepubs/007904875/utilities/xcu_chap02.html#tag_02_09_01_01>.  */
@@ -1335,10 +1399,9 @@ start_child (const char *exec_file, char **exec_argv,
 
 #ifdef HAVE_START_CHILD
 static SCM
-scm_open_process (SCM mode, SCM prog, SCM args)
-#define FUNC_NAME "open-process"
+scm_piped_process (SCM prog, SCM args, SCM from, SCM to)
+#define FUNC_NAME "piped-process"
 {
-  long mode_bits;
   int reading, writing;
   int c2p[2]; /* Child to parent.  */
   int p2c[2]; /* Parent to child.  */
@@ -1346,44 +1409,27 @@ scm_open_process (SCM mode, SCM prog, SCM args)
   int pid;
   char *exec_file;
   char **exec_argv;
-  SCM read_port = SCM_BOOL_F, write_port = SCM_BOOL_F;
 
   exec_file = scm_to_locale_string (prog);
   exec_argv = scm_i_allocate_string_pointers (scm_cons (prog, args));
 
-  mode_bits = scm_i_mode_bits (mode);
-  reading = mode_bits & SCM_RDNG;
-  writing = mode_bits & SCM_WRTNG;
+  reading = scm_is_pair (from);
+  writing = scm_is_pair (to);
 
   if (reading)
     {
-      if (pipe (c2p))
-        {
-          int errno_save = errno;
-          free (exec_file);
-          errno = errno_save;
-          SCM_SYSERROR;
-        }
+      c2p[0] = scm_to_int (scm_car (from));
+      c2p[1] = scm_to_int (scm_cdr (from));
       out = c2p[1];
     }
-  
+
   if (writing)
     {
-      if (pipe (p2c))
-        {
-          int errno_save = errno;
-          free (exec_file);
-          if (reading)
-            {
-              close (c2p[0]);
-              close (c2p[1]);
-            }
-          errno = errno_save;
-          SCM_SYSERROR;
-        }
+      p2c[0] = scm_to_int (scm_car (to));
+      p2c[1] = scm_to_int (scm_cdr (to));
       in = p2c[0];
     }
-  
+
   {
     SCM port;
 
@@ -1416,25 +1462,12 @@ scm_open_process (SCM mode, SCM prog, SCM args)
       SCM_SYSERROR;
     }
 
-  /* There is no sense in catching errors on close().  */
   if (reading)
-    {
-      close (c2p[1]);
-      read_port = scm_i_fdes_to_port (c2p[0], scm_mode_bits ("r0"),
-                                      sym_read_pipe,
-                                      SCM_FPORT_OPTION_NOT_SEEKABLE);
-    }
+    close (c2p[1]);
   if (writing)
-    {
-      close (p2c[0]);
-      write_port = scm_i_fdes_to_port (p2c[1], scm_mode_bits ("w0"),
-                                       sym_write_pipe,
-                                       SCM_FPORT_OPTION_NOT_SEEKABLE);
-    }
+    close (p2c[0]);
 
-  return scm_values (scm_list_3 (read_port,
-                                 write_port,
-                                 scm_from_int (pid)));
+  return scm_from_int (pid);
 }
 #undef FUNC_NAME
 
@@ -1479,8 +1512,8 @@ SCM_DEFINE (scm_system_star, "system*", 0, 0, 1,
 "Example: (system* \"echo\" \"foo\" \"bar\")")
 #define FUNC_NAME s_scm_system_star
 {
-  SCM prog, res;
-  int pid, status, wait_result;
+  SCM prog, pid;
+  int status, wait_result;
 
   if (scm_is_null (args))
     SCM_WRONG_NUM_ARGS ();
@@ -1490,17 +1523,16 @@ SCM_DEFINE (scm_system_star, "system*", 0, 0, 1,
   scm_dynwind_begin (0);
   /* Make sure the child can't kill us (as per normal system call).  */
   scm_dynwind_sigaction (SIGINT,
-                         scm_from_uintptr_t ((scm_t_uintptr) SIG_IGN),
+                         scm_from_uintptr_t ((uintptr_t) SIG_IGN),
                          SCM_UNDEFINED);
 #ifdef SIGQUIT
   scm_dynwind_sigaction (SIGQUIT,
-                         scm_from_uintptr_t ((scm_t_uintptr) SIG_IGN),
+                         scm_from_uintptr_t ((uintptr_t) SIG_IGN),
                          SCM_UNDEFINED);
 #endif
 
-  res = scm_open_process (scm_nullstr, prog, args);
-  pid = scm_to_int (scm_c_value_ref (res, 2));
-  SCM_SYSCALL (wait_result = waitpid (pid, &status, 0));
+  pid = scm_piped_process (prog, args, SCM_UNDEFINED, SCM_UNDEFINED);
+  SCM_SYSCALL (wait_result = waitpid (scm_to_int (pid), &status, 0));
   if (wait_result == -1)
     SCM_SYSERROR;
 
@@ -1557,6 +1589,8 @@ SCM_DEFINE (scm_environ, "environ", 0, 1, 0,
 }
 #undef FUNC_NAME
 
+#if (SCM_ENABLE_DEPRECATED == 1)
+#ifdef ENABLE_TMPNAM
 #ifdef L_tmpnam
 
 SCM_DEFINE (scm_tmpnam, "tmpnam", 0, 0, 0,
@@ -1565,12 +1599,15 @@ SCM_DEFINE (scm_tmpnam, "tmpnam", 0, 0, 0,
 	    "existing file.  However there is no guarantee that another\n"
 	    "process will not create the file after @code{tmpnam} is called.\n"
 	    "Care should be taken if opening the file, e.g., use the\n"
-	    "@code{O_EXCL} open flag or use @code{mkstemp!} instead.")
+	    "@code{O_EXCL} open flag or use @code{mkstemp} instead.")
 #define FUNC_NAME s_scm_tmpnam
 {
   char name[L_tmpnam];
   char *rv;
 
+  scm_c_issue_deprecation_warning
+      ("Use of tmpnam is deprecated.  Use mkstemp instead.");
+
   SCM_SYSCALL (rv = tmpnam (name));
   if (rv == NULL)
     /* not SCM_SYSERROR since errno probably not set.  */
@@ -1579,6 +1616,8 @@ SCM_DEFINE (scm_tmpnam, "tmpnam", 0, 0, 0,
 }
 #undef FUNC_NAME
 
+#endif
+#endif
 #endif
 
 SCM_DEFINE (scm_tmpfile, "tmpfile", 0, 0, 0,
@@ -1624,7 +1663,10 @@ SCM_DEFINE (scm_utime, "utime", 1, 5, 0,
 	    "(utime \"foo\" (- (current-time) 3600))\n"
 	    "@end lisp\n"
 	    "will set the access time to one hour in the past and the\n"
-	    "modification time to the current time.")
+	    "modification time to the current time.\n\n"
+            "Last, @var{flags} may be either @code{0} or the\n"
+            "@code{AT_SYMLINK_NOFOLLOW} constant, to set the time of\n"
+            "@var{pathname} even if it is a symbolic link.\n")
 #define FUNC_NAME s_scm_utime
 {
   int rv;
@@ -2092,7 +2134,7 @@ cpu_set_to_bitvector (const cpu_set_t *cs)
     {
       if (CPU_ISSET (cpu, cs))
 	/* XXX: This is inefficient but avoids code duplication.  */
-	scm_c_bitvector_set_x (bv, cpu, SCM_BOOL_T);
+	scm_c_bitvector_set_bit_x (bv, cpu);
     }
 
   return bv;
@@ -2130,7 +2172,7 @@ SCM_DEFINE (scm_setaffinity, "setaffinity", 2, 0, 0,
 {
   cpu_set_t cs;
   scm_t_array_handle handle;
-  const scm_t_uint32 *c_mask;
+  const uint32_t *c_mask;
   size_t len, off, cpu;
   ssize_t inc;
   int err;
@@ -2337,7 +2379,7 @@ SCM_DEFINE (scm_gethostname, "gethostname", 0, 0, 0,
 static void
 scm_init_popen (void)
 {
-  scm_c_define_gsubr ("open-process", 2, 0, 1, scm_open_process);
+  scm_c_define_gsubr ("piped-process", 2, 2, 0, scm_piped_process);
 }
 #endif /* HAVE_START_CHILD */
 
@@ -2433,8 +2475,21 @@ scm_init_posix ()
   scm_c_define ("LOCK_NB", scm_from_int (LOCK_NB));
 #endif
 
-#include "libguile/cpp-SIG.c"
-#include "libguile/posix.x"
+#ifdef AT_SYMLINK_NOFOLLOW
+  scm_c_define ("AT_SYMLINK_NOFOLLOW", scm_from_int (AT_SYMLINK_NOFOLLOW));
+#endif
+#ifdef AT_SYMLINK_FOLLOW
+  scm_c_define ("AT_SYMLINK_FOLLOW", scm_from_int (AT_SYMLINK_FOLLOW));
+#endif
+#ifdef AT_NO_AUTOMOUNT
+  scm_c_define ("AT_NO_AUTOMOUNT", scm_from_int (AT_NO_AUTOMOUNT));
+#endif
+#ifdef AT_EMPTY_PATH
+  scm_c_define ("AT_EMPTY_PATH", scm_from_int (AT_EMPTY_PATH));
+#endif
+
+#include "cpp-SIG.c"
+#include "posix.x"
 
 #ifdef HAVE_FORK
   scm_add_feature ("fork");
@@ -2447,9 +2502,3 @@ scm_init_posix ()
 			    NULL);
 #endif /* HAVE_START_CHILD */
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
