@@ -1,4 +1,4 @@
-/* Copyright 1995-1996,1998-2001,2006,2008-2012,2014,2018-2019
+/* Copyright 1995-1996,1998-2001,2006,2008-2012,2014,2018-2020
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -43,7 +43,8 @@
 
 #define SCM_VALIDATE_MUTABLE_VECTOR(pos, v)                             \
   do {                                                                  \
-    SCM_ASSERT (SCM_I_IS_MUTABLE_VECTOR (v), v, pos, FUNC_NAME);        \
+    SCM_ASSERT_TYPE (SCM_I_IS_MUTABLE_VECTOR (v), v, pos, FUNC_NAME,    \
+                     "mutable vector");                                 \
   } while (0)
 
 
@@ -240,6 +241,11 @@ SCM_DEFINE (scm_make_vector, "make-vector", 1, 1, 0,
 }
 #undef FUNC_NAME
 
+static SCM
+make_vector (size_t size)
+{
+  return scm_words ((size << 8) | scm_tc7_vector, size + 1);
+}
 
 SCM
 scm_c_make_vector (size_t k, SCM fill)
@@ -250,8 +256,7 @@ scm_c_make_vector (size_t k, SCM fill)
 
   SCM_ASSERT_RANGE (1, scm_from_size_t (k), k <= VECTOR_MAX_LENGTH);
 
-  vector = scm_words ((k << 8) | scm_tc7_vector, k + 1);
-
+  vector = make_vector (k);
   for (j = 0; j < k; ++j)
     SCM_SIMPLE_VECTOR_SET (vector, j, fill);
 
@@ -272,7 +277,7 @@ SCM_DEFINE (scm_vector_copy, "vector-copy", 1, 0, 0,
 
   src = scm_vector_elements (vec, &handle, &len, &inc);
 
-  result = scm_c_make_vector (len, SCM_UNDEFINED);
+  result = make_vector (len);
   dst = SCM_I_VECTOR_WELTS (result);
   for (i = 0; i < len; i++, src += inc)
     dst[i] = *src;
@@ -311,23 +316,43 @@ SCM_DEFINE (scm_vector_to_list, "vector->list", 1, 0, 0,
 }
 #undef FUNC_NAME
 
+static SCM scm_vector_fill_partial_x (SCM vec, SCM fill, SCM start, SCM end);
 
-SCM_DEFINE (scm_vector_fill_x, "vector-fill!", 2, 0, 0,
-            (SCM v, SCM fill),
-	    "Store @var{fill} in every position of @var{vector}.  The value\n"
-	    "returned by @code{vector-fill!} is unspecified.")
+SCM_DEFINE_STATIC (scm_vector_fill_partial_x, "vector-fill!", 2, 2, 0,
+            (SCM vec, SCM fill, SCM start, SCM end),
+            "Assign the value of every location in vector @var{vec} between\n"
+            "@var{start} and @var{end} to @var{fill}.  @var{start} defaults\n"
+            "to 0 and @var{end} defaults to the length of @var{vec}.  The value\n"
+            "returned by @code{vector-fill!} is unspecified.")
+#define FUNC_NAME s_scm_vector_fill_partial_x
+{
+  SCM_VALIDATE_MUTABLE_VECTOR(1, vec);
+
+  SCM *data;
+  size_t i = 0;
+  size_t len = SCM_I_VECTOR_LENGTH (vec);
+
+  data = SCM_I_VECTOR_WELTS (vec);
+
+  if (!SCM_UNBNDP (start))
+    i = scm_to_unsigned_integer (start, 0, len);
+
+  if (!SCM_UNBNDP (end))
+    len = scm_to_unsigned_integer (end, i, len);
+
+  for (; i < len; ++i)
+    data[i] = fill;
+
+  return SCM_UNSPECIFIED;
+}
+#undef FUNC_NAME
+
+
+SCM
+scm_vector_fill_x (SCM vec, SCM fill)
 #define FUNC_NAME s_scm_vector_fill_x
 {
-  scm_t_array_handle handle;
-  SCM *data;
-  size_t i, len;
-  ssize_t inc;
-
-  data = scm_vector_writable_elements (v, &handle, &len, &inc);
-  for (i = 0; i < len; i += inc)
-    data[i] = fill;
-  scm_array_handle_release (&handle);
-  return SCM_UNSPECIFIED;
+  return scm_vector_fill_partial_x (vec, fill, SCM_UNDEFINED, SCM_UNDEFINED);
 }
 #undef FUNC_NAME
 
