@@ -1,6 +1,6 @@
 ;;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013, 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2021 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,6 @@
   #:use-module (language cps)
   #:use-module (language cps slot-allocation)
   #:use-module (language cps utils)
-  #:use-module (language cps closure-conversion)
-  #:use-module (language cps loop-instrumentation)
-  #:use-module (language cps optimize)
-  #:use-module (language cps reify-primitives)
-  #:use-module (language cps renumber)
-  #:use-module (language cps split-rec)
   #:use-module (language cps intmap)
   #:use-module (language cps intset)
   #:use-module (system vm assembler)
@@ -125,7 +119,8 @@
          (for-each (match-lambda
                     ((src . dst) (emit-mov asm (from-sp dst) (from-sp src))))
                    (lookup-parallel-moves label allocation))
-         (maybe-reset-frame (1+ (length args)))
+         (let ((nclosure (if proc 1 0)))
+           (maybe-reset-frame (+ nclosure (length args))))
          (emit-handle-interrupts asm)
          (emit-tail-call-label asm k))
         (($ $values args)
@@ -161,6 +156,12 @@
          (emit-allocate-words asm (from-sp dst) (from-sp (slot nfields))))
         (($ $primcall 'allocate-words/immediate (annotation . nfields))
          (emit-allocate-words/immediate asm (from-sp dst) nfields))
+        (($ $primcall 'allocate-pointerless-words annotation (nfields))
+         (emit-allocate-pointerless-words asm (from-sp dst)
+                                       (from-sp (slot nfields))))
+        (($ $primcall 'allocate-pointerless-words/immediate
+            (annotation . nfields))
+         (emit-allocate-pointerless-words/immediate asm (from-sp dst) nfields))
         (($ $primcall 'scm-ref annotation (obj idx))
          (emit-scm-ref asm (from-sp dst) (from-sp (slot obj))
                        (from-sp (slot idx))))
@@ -185,8 +186,21 @@
          (emit-cache-ref asm (from-sp dst) key))
         (($ $primcall 'resolve-module public? (name))
          (emit-resolve-module asm (from-sp dst) (from-sp (slot name)) public?))
+        (($ $primcall 'module-variable #f (mod name))
+         (emit-module-variable asm (from-sp dst) (from-sp (slot mod))
+                               (from-sp (slot name))))
         (($ $primcall 'lookup #f (mod name))
-         (emit-lookup asm (from-sp dst) (from-sp (slot mod)) (from-sp (slot name))))
+         (emit-lookup asm (from-sp dst) (from-sp (slot mod))
+                      (from-sp (slot name))))
+        (($ $primcall 'lookup-bound #f (mod name))
+         (emit-lookup-bound asm (from-sp dst) (from-sp (slot mod))
+                            (from-sp (slot name))))
+        (($ $primcall 'lookup-bound-public (mod name) ())
+         (let ((name (symbol->string name)))
+           (emit-lookup-bound-public asm (from-sp dst) mod name)))
+        (($ $primcall 'lookup-bound-private (mod name) ())
+         (let ((name (symbol->string name)))
+           (emit-lookup-bound-private asm (from-sp dst) mod name)))
         (($ $primcall 'add/immediate y (x))
          (emit-add/immediate asm (from-sp dst) (from-sp (slot x)) y))
         (($ $primcall 'sub/immediate y (x))
@@ -379,6 +393,24 @@
         (#('throw/value+data param (val))
          (emit-throw/value+data asm (from-sp (slot val)) param))))
 
+    (define (emit-parallel-moves-after-return-and-reset-frame label nlocals)
+      (let lp ((moves (lookup-parallel-moves label allocation))
+               (reset-frame? #f))
+        (cond
+         ((and (not reset-frame?)
+               (and-map (match-lambda
+                         ((src . dst)
+                          (and (< src nlocals) (< dst nlocals))))
+                        moves))
+          (emit-reset-frame asm nlocals)
+          (lp moves #t))
+         (else
+          (match moves
+            (() #t)
+            (((src . dst) . moves)
+             (emit-fmov asm dst src)
+             (lp moves reset-frame?)))))))
+
     (define (compile-prompt label k kh escape? tag)
       (match (intmap-ref cps kh)
         (($ $kreceive ($ $arity req () rest () #f) khandler-body)
@@ -396,10 +428,7 @@
                         (($ $kargs names (_ ... rest))
                          (maybe-slot rest))))
              (emit-bind-rest asm (+ proc-slot nreq)))
-           (for-each (match-lambda
-                      ((src . dst) (emit-fmov asm dst src)))
-                     (lookup-parallel-moves kh allocation))
-           (emit-reset-frame asm frame-size)
+           (emit-parallel-moves-after-return-and-reset-frame kh frame-size)
            (emit-j asm (forward-label khandler-body))))))
 
     (define (compile-values label exp syms)
@@ -453,13 +482,8 @@
         (#('fixnum? #f (a)) (unary emit-fixnum? a))
         (#('heap-object? #f (a)) (unary emit-heap-object? a))
         (#('char? #f (a)) (unary emit-char? a))
-        (#('eq-false? #f (a)) (unary emit-eq-false? a))
-        (#('eq-nil? #f (a)) (unary emit-eq-nil? a))
-        (#('eq-null? #f (a)) (unary emit-eq-null? a))
-        (#('eq-true? #f (a)) (unary emit-eq-true? a))
-        (#('unspecified? #f (a)) (unary emit-unspecified? a))
+        (#('eq-constant? imm (a)) (binary-test/imm emit-eq-immediate? a imm))
         (#('undefined? #f (a)) (unary emit-undefined? a))
-        (#('eof-object? #f (a)) (unary emit-eof-object? a))
         (#('null? #f (a)) (unary emit-null? a))
         (#('false? #f (a)) (unary emit-false? a))
         (#('nil? #f (a)) (unary emit-nil? a))
@@ -519,7 +543,8 @@
     (define (compile-trunc label k exp nreq rest-var)
       (define (do-call proc args emit-call)
         (let* ((proc-slot (lookup-call-proc-slot label allocation))
-               (nargs (1+ (length args)))
+               (nclosure (if proc 1 0))
+               (nargs (+ nclosure (length args)))
                (arg-slots (map (lambda (x) (+ x proc-slot)) (iota nargs))))
           (for-each (match-lambda
                      ((src . dst) (emit-mov asm (from-sp dst) (from-sp src))))
@@ -542,10 +567,7 @@
               (emit-receive-values asm proc-slot (->bool rest-var) nreq))
             (when (and rest-var (maybe-slot rest-var))
               (emit-bind-rest asm (+ proc-slot nreq)))
-            (for-each (match-lambda
-                       ((src . dst) (emit-fmov asm dst src)))
-                      (lookup-parallel-moves k allocation))
-            (emit-reset-frame asm frame-size)))))
+            (emit-parallel-moves-after-return-and-reset-frame k frame-size)))))
       (match exp
         (($ $call proc args)
          (do-call proc args
@@ -605,6 +627,12 @@
          (compile-test label (skip-elided-conts (1+ label))
                        (forward-label kf) (forward-label kt)
                        op param args))
+        (($ $switch kf kt* src arg)
+         (when src
+           (emit-source asm src))
+         (emit-jtable asm (from-sp (slot arg))
+                      (list->vector (map forward-label
+                                         (append kt* (list kf))))))
         (($ $prompt k kh src escape? tag)
          (when src
            (emit-source asm src))
@@ -616,14 +644,26 @@
 
     (define (compile-cont label cont)
       (match cont
-        (($ $kfun src meta self tail clause)
+        (($ $kfun src meta self tail entry)
          (when src
            (emit-source asm src))
-         (emit-begin-program asm label meta))
+         (emit-begin-program asm label meta)
+         (match (intmap-ref cps entry)
+           (($ $kclause)
+            ;; Leave arity handling to the dispatcher.
+            #t)
+           (($ $kargs names vars _)
+            ;; Otherwise the $kfun continues to the $kargs directly,
+            ;; without any arity checking, so we begin the arity here.
+            (emit-begin-unchecked-arity asm (->bool self) names frame-size)
+            (when self
+              (emit-definition asm 'closure 0 'scm)))))
         (($ $kclause ($ $arity req opt rest kw allow-other-keys?) body alt)
          (let ((first? (match (intmap-ref cps (1- label))
                          (($ $kfun) #t)
                          (_ #f)))
+               (has-closure? (match (intmap-ref cps (intmap-next cps))
+                               (($ $kfun src meta self tail) (->bool self))))
                (kw-indices (map (match-lambda
                                  ((key name sym)
                                   (cons key (lookup-slot sym allocation))))
@@ -631,10 +671,11 @@
            (unless first?
              (emit-end-arity asm))
            (emit-label asm label)
-           (emit-begin-kw-arity asm req opt rest kw-indices allow-other-keys?
-                                frame-size alt)
-           ;; All arities define a closure binding in slot 0.
-           (emit-definition asm 'closure 0 'scm)
+           (emit-begin-kw-arity asm has-closure? req opt rest kw-indices
+                                allow-other-keys? frame-size alt)
+           (when has-closure?
+             ;; Most arities define a closure binding in slot 0.
+             (emit-definition asm 'closure 0 'scm))
            ;; Usually we just fall through, but it could be the body is
            ;; contified into another clause.
            (let ((body (forward-label body)))
@@ -657,7 +698,7 @@
 
     (intmap-for-each compile-cont cps)))
 
-(define (emit-bytecode exp env opts)
+(define (compile-bytecode exp env opts)
   (let ((asm (make-assembler)))
     (intmap-for-each (lambda (kfun body)
                        (compile-function (intmap-select exp body) asm opts))
@@ -665,20 +706,3 @@
     (values (link-assembly asm #:page-aligned? (kw-arg-ref opts #:to-file? #f))
             env
             env)))
-
-(define (lower-cps exp opts)
-  ;; FIXME: For now the closure conversion pass relies on $rec instances
-  ;; being separated into SCCs.  We should fix this to not be the case,
-  ;; and instead move the split-rec pass back to
-  ;; optimize-higher-order-cps.
-  (set! exp (split-rec exp))
-  (set! exp (optimize-higher-order-cps exp opts))
-  (set! exp (convert-closures exp))
-  (set! exp (optimize-first-order-cps exp opts))
-  (set! exp (reify-primitives exp))
-  (set! exp (add-loop-instrumentation exp))
-  (renumber exp))
-
-(define (compile-bytecode exp env opts)
-  (set! exp (lower-cps exp opts))
-  (emit-bytecode exp env opts))
