@@ -1,6 +1,6 @@
 ;;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2021 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -201,34 +201,14 @@
   (wrap-unary cps k src 's64->u64 'u64->s64 'ulsh/immediate param a))
 
 (define (reify-lookup cps src mod-var name assert-bound? have-var)
-  (define (%lookup cps kbad k src mod-var name-var var assert-bound?)
-    (if assert-bound?
-        (with-cps cps
-          (letv val)
-          (letk kcheck
-                ($kargs ('val) (val)
-                  ($branch k kbad src 'undefined? #f (val))))
-          (letk kref
-                ($kargs () ()
-                  ($continue kcheck src
-                    ($primcall 'scm-ref/immediate '(box . 1) (var)))))
-          ($ (%lookup kbad kref src mod-var name-var var #f)))
-        (with-cps cps
-          (letk kres
-                ($kargs ('var) (var)
-                  ($branch kbad k src 'heap-object? #f (var))))
-          (build-term
-            ($continue kres src
-              ($primcall 'lookup #f (mod-var name-var)))))))
-  (define %unbound
-    #(unbound-variable #f "Unbound variable: ~S"))
   (with-cps cps
     (letv name-var var)
-    (let$ good (have-var var))
-    (letk kgood ($kargs () () ,good))
-    (letk kbad ($kargs () () ($throw src 'throw/value %unbound (name-var))))
-    (let$ body (%lookup kbad kgood src mod-var name-var var assert-bound?))
-    (letk klookup ($kargs ('name) (name-var) ,body))
+    (let$ body (have-var var))
+    (letk kres ($kargs ('var) (var) ,body))
+    (letk klookup ($kargs ('name) (name-var)
+                    ($continue kres src
+                      ($primcall (if assert-bound? 'lookup-bound 'lookup) #f
+                                 (mod-var name-var)))))
     (build-term ($continue klookup src ($const name)))))
 
 (define (reify-resolve-module cps k src module public?)
@@ -243,8 +223,30 @@
 
 (define-ephemeral (cached-module-box cps k src param)
   (match param
+    ((module name public? #t)
+     (let ((cache-key param))
+       (with-cps cps
+         (letv cached var)
+         (letk k* ($kargs () () ($continue k src ($values (var)))))
+         (letk kcache ($kargs ('var) (var)
+                        ($continue k* src
+                          ($primcall 'cache-set! cache-key (var)))))
+         (letk kinit ($kargs () ()
+                       ($continue kcache src
+                         ($primcall (if public?
+                                        'lookup-bound-public
+                                        'lookup-bound-private)
+                                    (list module name) ()))))
+         (letk kok ($kargs () ()
+                     ($continue k src ($values (cached)))))
+         (letk ktest
+               ($kargs ('cached) (cached)
+                 ($branch kinit kok src 'heap-object? #f (cached))))
+         (build-term
+           ($continue ktest src
+             ($primcall 'cache-ref cache-key ()))))))
     ((module name public? bound?)
-     (let ((cache-key (cons module name)))
+     (let ((cache-key param))
        (with-cps cps
          (letv mod cached)
          (let$ lookup
@@ -314,6 +316,7 @@
       quo
       rem
       mod
+      inexact
       sqrt
       abs
       floor
@@ -353,7 +356,9 @@
       push-dynamic-state pop-dynamic-state
       lsh rsh lsh/immediate rsh/immediate
       cache-ref cache-set!
-      resolve-module lookup define! current-module))
+      current-module resolve-module
+      module-variable define!
+      lookup lookup-bound lookup-bound-public lookup-bound-private))
   (let ((table (make-hash-table)))
     (for-each
      (match-lambda ((inst . _) (hashq-set! table inst #t)))
@@ -416,7 +421,7 @@
                         ($primcall 'load-u64 %tc16-flonum ()))))
          (setk label ($kargs names vars
                        ($continue ktag0 src
-                         ($primcall 'allocate-words/immediate
+                         ($primcall 'allocate-pointerless-words/immediate
                                     `(flonum . ,(match (target-word-size)
                                                   (4 4)
                                                   (8 2)))
@@ -507,7 +512,14 @@
               ;; ((ulsh/immediate (u6? y) x) (ulsh x y))
               (_
                (match (cons name args)
-                 (('allocate-words/immediate)
+                 (((or 'allocate-words/immediate
+                       'allocate-pointerless-words/immediate))
+                  (define op
+                    (match name
+                      ('allocate-words/immediate
+                       'allocate-words)
+                      ('allocate-pointerless-words/immediate
+                       'allocate-pointerless-words)))
                   (match param
                     ((ann . n)
                      (if (u8? n)
@@ -516,7 +528,7 @@
                            (letv n*)
                            (letk kop ($kargs ('n) (n*)
                                        ($continue k src
-                                         ($primcall 'allocate-words ann (n*)))))
+                                         ($primcall op ann (n*)))))
                            (setk label ($kargs names vars
                                          ($continue kop src
                                            ($primcall 'load-u64 n ())))))))))
@@ -568,8 +580,19 @@
          (define (u11? val) (<= 0 val #x7ff))
          (define (u12? val) (<= 0 val #xfff))
          (define (s12? val) (<= (- #x800) val #x7ff))
+         (define (imm16? val)
+           (and=> (scm->immediate-bits val)
+                  (lambda (bits)
+                    (truncate-bits bits 16 #t))))
+         (define (load-u64 k param)
+           (build-term ($continue k src ($primcall 'load-u64 param ()))))
+         (define (load-s64 k param)
+           (build-term ($continue k src ($primcall 'load-s64 param ()))))
+         (define (load-const k param)
+           (build-term ($continue k src ($const param))))
+
          (define-syntax-rule (reify-constants ((op (pred? c) in ...)
-                                               wrap-op (op* out ...))
+                                               wrap (op* out ...))
                                               ...
                                               (_ default))
            (match name
@@ -584,9 +607,7 @@
                              ($kargs ('c) (c)
                                ($branch kf kt src 'op* #f (out ...))))
                        (setk label
-                             ($kargs names vars
-                               ($continue kconst src
-                                 ($primcall 'wrap-op param ())))))))))
+                             ($kargs names vars ,(wrap kconst param))))))))
              ...
              (_ default)))
          (reify-constants
@@ -596,6 +617,7 @@
           ((s64-imm-= (s12? b) a) load-s64 (s64-= a b))
           ((s64-imm-< (s12? b) a) load-s64 (s64-< a b))
           ((imm-s64-< (s12? a) b) load-s64 (s64-< a b))
+          ((eq-constant? (imm16? b) a) load-const (eq? a b))
           (_ cps))))
       (($ $kargs names vars ($ $continue k src ($ $call proc args)))
        (with-cps cps
