@@ -1,6 +1,6 @@
 ;;;; test-suite/lib.scm --- generic support for testing
-;;;; Copyright (C) 1999, 2000, 2001, 2004, 2006, 2007, 2009, 2010,
-;;;;   2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+;;;; Copyright (C) 1999-2001,2004,2006-2007,2009-2014,2016,2018,2020
+;;;;   Free Software Foundation, Inc.
 ;;;;
 ;;;; This program is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -39,7 +39,6 @@
  exception:string-contains-nul
  exception:read-error
  exception:null-pointer-error
- exception:vm-error
 
  ;; Reporting passes and failures.
  run-test
@@ -294,8 +293,6 @@
   (cons 'read-error "^.*$"))
 (define exception:null-pointer-error
   (cons 'null-pointer-error "^.*$"))
-(define exception:vm-error
-  (cons 'vm-error "^.*$"))
 
 ;; as per throw in scm_to_locale_stringn()
 (define exception:string-contains-nul
@@ -469,20 +466,31 @@
      (c&e (pass-if "[unnamed test]" exp)))
     ((_ (pass-if test-name exp))
      (begin (pass-if (string-append test-name " (eval)")
-                     (primitive-eval 'exp))
-            (pass-if (string-append test-name " (compile)")
-                     (compile 'exp #:to 'value #:env (current-module)))))
+              (primitive-eval 'exp))
+            (pass-if (string-append test-name " (compile -O0)")
+              (compile 'exp #:to 'value #:env (current-module)
+                       #:optimization-level 0))
+            (pass-if (string-append test-name " (compile -O2)")
+              (compile 'exp #:to 'value #:env (current-module)
+                       #:optimization-level 2))))
     ((_ (pass-if-equal test-name val exp))
      (begin (pass-if-equal (string-append test-name " (eval)") val
               (primitive-eval 'exp))
-            (pass-if-equal (string-append test-name " (compile)") val
-              (compile 'exp #:to 'value #:env (current-module)))))
+            (pass-if-equal (string-append test-name " (compile -O0)") val
+              (compile 'exp #:to 'value #:env (current-module)
+                       #:optimization-level 0))
+            (pass-if-equal (string-append test-name " (compile -O2)") val
+              (compile 'exp #:to 'value #:env (current-module)
+                       #:optimization-level 2))))
     ((_ (pass-if-exception test-name exc exp))
-     (begin (pass-if-exception (string-append test-name " (eval)")
-                               exc (primitive-eval 'exp))
-            (pass-if-exception (string-append test-name " (compile)")
-                               exc (compile 'exp #:to 'value
-                                            #:env (current-module)))))))
+     (begin (pass-if-exception (string-append test-name " (eval)") exc
+              (primitive-eval 'exp))
+            (pass-if-exception (string-append test-name " (compile -O0)") exc
+              (compile 'exp #:to 'value #:env (current-module)
+                       #:optimization-level 0))
+            (pass-if-exception (string-append test-name " (compile -O2)") exc
+              (compile 'exp #:to 'value #:env (current-module)
+                       #:optimization-level 2))))))
 
 ;;; (with-test-prefix/c&e PREFIX BODY ...)
 ;;; Same as `with-test-prefix', but the enclosed tests are run both with
